/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.common.provider;

import java.util.function.Consumer;

import org.simantics.db.AsyncReadGraph;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;

/**
 * An interface for externalizing the logic of retrieving an arbitrary resource
 * for some purpose.
 * 
 * <p>
 * A {@link ReadGraph} is provided to allow implementations to browse the graph
 * in order to retrieve a resource.
 * </p>
 * 
 * <p>
 * This interface has a synchronously operating counterpart, {@link ResourceProvider}.
 * </p>
 * 
 * @see ResourceProviders
 * @see ResourceProvider
 * 
 * @author Tuukka Lehtonen
 */
public interface AsyncResourceProvider {

    /**
     * @param graph an asynchronous graph for retrieving the resource to return
     * @param callback a callback that is invoked with the provided resource
     */
    void get(AsyncReadGraph graph, Consumer<Resource> callback);

}
