/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.common.request;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.common.utils.NameUtils;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.exception.NoSingleResultException;

/**
 * @author Tuukka Lehtonen
 */
public final class SingleObjectWithType extends ResourceRead3<Resource> {

    public SingleObjectWithType(Resource resource, Resource predicate, Resource type) {
        super(resource, predicate, type);
    }

    @Override
    public Resource perform(ReadGraph graph) throws DatabaseException {
        Resource result = null;
        int resultCount = 0;
        for (Resource object : graph.getObjects(resource, resource2)) {
            if (graph.isInstanceOf(object, resource3)) {
                result = object;
                resultCount++;
            }
        }
        
        if (resultCount != 1) {
            String reason = resultCount == 0 ? "No objects for relation " : "Multiple objects for relation ";
            throw new NoSingleResultException(reason
                    + NameUtils.getSafeName(graph, resource2) + " with type "
                    + NameUtils.getSafeName(graph, resource3) + " at "
                    + NameUtils.getSafeName(graph, resource), resultCount);
        }

        return result;
    }

}
